import os

from sqlalchemy import Column, BigInteger, String, Table, ForeignKey
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship
from sqlalchemy_serializer import SerializerMixin

from database import Base
from search.searchable_mixin import SearchableMixin

tag_to_file = Table(
    'tag_to_file',
    Base.metadata,
    Column('tag_id', BigInteger, ForeignKey('tags.id'), primary_key=True),
    Column('file_id', BigInteger, ForeignKey('files.id'), primary_key=True)
)


class FileModel(Base, SerializerMixin, SearchableMixin):
    __searchable__ = ['__tags__']
    __tablename__ = 'files'
    id = Column(BigInteger, primary_key=True)
    __uuid__ = Column(UUID(as_uuid=True), index=True)
    filepath = Column(String(128))
    __tags__ = relationship(
        "TagModel",
        secondary=tag_to_file,
        backref="__files__"
    )

    def uuid(self):
        return str(self.__uuid__)

    def tags(self):
        return [tag.name for tag in self.__tags__]

    def file_url(self):
        return f'http://{os.environ.get("HOST", "localhost")}:{os.environ.get("PORT", "18080")}/' \
               f'{self.filepath}/{str(self.__uuid__)}.png'

    def thumbnail_url(self):
        return f'http://{os.environ.get("HOST", "localhost")}:{os.environ.get("PORT", "18080")}/' \
               f'{self.filepath}/thumbnail_{str(self.__uuid__)}.png'

    serialize_only = (
        "uuid",
        "file_url",
        "thumbnail_url",
        "tags"
    )

    def __init__(
            self, id=None, filepath=None, uuid=None):
        super().__init__()
        self.id = id
        self.filepath = filepath
        self.__uuid__ = uuid

    def __repr__(self):
        return '<File %s>' % self.__uuid__
